from __future__ import with_statement
import pytest
import py
import time
import sys
import eventlet
import detox
from py.builtin import print_
from eventlet.green.subprocess import Popen
from textwrap import dedent as d
from detox.proc import Detox
from _pytest.pytester import RunResult, getdecoded
import detox.main

pytest_plugins = "pytester"

def create_example1(tmpdir):
    tmpdir.join("setup.py").write(d("""
        from setuptools import setup

        def main():
            setup(
                name='example1',
                description='example1 project for testing detox',
                version='0.4',
                packages=['example1',],
            )
        if __name__ == '__main__':
            main()
    """))
    tmpdir.join("tox.ini").write(d("""
        [testenv:py]
    """))
    tmpdir.join("example1", "__init__.py").ensure()


def create_example2(tmpdir):
    tmpdir.join("tox.ini").write(d("""
        [tox]
        skipsdist = True

        [testenv:py]
    """))
    tmpdir.join("example2", "__init__.py").ensure()


def create_example3(tmpdir):
    tmpdir.join("tox.ini").write(d("""
        [tox]
        skipsdist = True

        [testenv]
        commands = python -c 'import time; time.sleep(1)'

        [testenv:py1]
        [testenv:py2]
    """))
    tmpdir.join("example3", "__init__.py").ensure()


def pytest_configure(config):
    config.addinivalue_line("markers", "example1: use example1 for setup")
    config.addinivalue_line("markers", "example2: use example2 for setup")
    config.addinivalue_line("markers", "timeout(N): stop test function "
        "after N seconds, throwing a Timeout.")

def pytest_funcarg__exampledir(request):
    tmpdir = request.getfuncargvalue("tmpdir")
    for x in dir(request.function):
        if x.startswith("example"):
            exampledir = tmpdir.mkdir(x)
            globals()["create_"+x](exampledir)
            print ("%s created at %s" %(x,exampledir))
            break
    else:
        raise request.LookupError("test function has example")
    return exampledir

def pytest_funcarg__detox(request):
    exampledir = request.getfuncargvalue("exampledir")
    old = exampledir.chdir()
    try:
        return Detox(detox.main.parse([]))
    finally:
        old.chdir()

def pytest_funcarg__cmd(request):
    exampledir = request.getfuncargvalue("exampledir")
    cmd = Cmd(exampledir, request)
    return cmd

class Cmd:
    def __init__(self, basedir, request):
        self.basedir = basedir
        self.tmpdir = basedir.mkdir(".cmdtmp")
        self.request = request

    def main(self, *args):
        self.basedir.chdir()
        return detox.main.main(args)

    def rundetox(self, *args):
        old = self.basedir.chdir()
        script = py.path.local.sysfind("detox")
        assert script, "could not find 'detox' script"
        return self._run(script, *args)

    def _run(self, *cmdargs):
        cmdargs = [str(x) for x in cmdargs]
        p1 = self.tmpdir.join("stdout")
        p2 = self.tmpdir.join("stderr")
        print_("running", cmdargs, "curdir=", py.path.local())
        f1 = p1.open("wb")
        f2 = p2.open("wb")
        now = time.time()
        popen = Popen(cmdargs, stdout=f1, stderr=f2,
            close_fds=(sys.platform != "win32"))
        ret = popen.wait()
        f1.close()
        f2.close()
        out = p1.read("rb")
        out = getdecoded(out).splitlines()
        err = p2.read("rb")
        err = getdecoded(err).splitlines()
        def dump_lines(lines, fp):
            try:
                for line in lines:
                    py.builtin.print_(line, file=fp)
            except UnicodeEncodeError:
                print("couldn't print to %s because of encoding" % (fp,))
        dump_lines(out, sys.stdout)
        dump_lines(err, sys.stderr)
        return RunResult(ret, out, err, time.time()-now)

@pytest.mark.tryfirst
def pytest_pyfunc_call(__multicall__, pyfuncitem):
    try:
        timeout = pyfuncitem.obj.timeout.args[0]
    except (AttributeError, IndexError):
        timeout = 5.0
    with eventlet.Timeout(timeout):
        return __multicall__.execute()

def test_pyfuncall():
    class MC:
        def execute(self):
            eventlet.sleep(5.0)
    class pyfuncitem:
        class obj:
            class timeout:
                args = [0.001]
    pytest.raises(eventlet.Timeout,
        lambda: pytest_pyfunc_call(MC(), pyfuncitem))

def test_hang(testdir):
    p = py.path.local(__file__).dirpath('conftest.py')
    p.copy(testdir.tmpdir.join(p.basename))
    t = testdir.makepyfile("""
        import pytest
        from eventlet.green import time
        @pytest.mark.timeout(0.01)
        def test_hang():
            time.sleep(3.0)
    """)
    result = testdir.runpytest()
    assert "failed to timeout" not in result.stdout.str()
    result.stdout.fnmatch_lines(["*Timeout: 0.01*"])



from __future__ import print_function

import sys

from detox import __version__
from detox.proc import Detox


def parse(args):
    from tox.session import prepare
    return prepare(args)


def main(args=None):
    if args is None:
        args = sys.argv[1:]
    if args and args[0] == "--version":
        print("detox {}".format(__version__))
        return
    config = parse(args)
    detox = Detox(config)
    detox.startloopreport()
    retcode = detox.runtestsmulti(config.envlist)
    raise SystemExit(retcode)



from setuptools import setup

long_description = """
What is detox?
==========================

detox is the distributed version of "tox".  It makes efficient use of multiple
CPUs by running all possible activities in parallel.  It has the same options
and configuration that tox has so after installation can just run::

    detox

in the same way and with the same options with which you would run
``tox``, see the `tox home page`_ for more info.

Please file issues as "tox" issues using the "detox" label:

    https://github.com/tox-dev/tox/issues

.. note::

    detox runs on python2.7 and python3.4+ (but supports creation of
    all environments supported of the underlying "tox" command)

.. _`tox home page`: http://tox.readthedocs.org
"""


def main():
    setup(
        name='detox',
        description='distributing activities of the tox tool',
        long_description=long_description,
        version='0.14.post1',  # Note: keep in sync with detox/__init__.py
        url='https://github.com/tox-dev/detox',
        license='MIT',
        platforms=['unix', 'linux', 'osx', 'cygwin', 'win32'],
        author='holger krekel',
        author_email='holger@merlinux.eu',
        classifiers=['Development Status :: 4 - Beta',
                     'Framework :: tox',
                     'Intended Audience :: Developers',
                     'License :: OSI Approved :: MIT License',
                     'Operating System :: POSIX',
                     'Operating System :: Microsoft :: Windows',
                     'Operating System :: MacOS :: MacOS X',
                     'Programming Language :: Python :: 2.7',
                     'Programming Language :: Python :: 3.4',
                     'Programming Language :: Python :: 3.5',
                     'Programming Language :: Python :: 3.6',
                     'Programming Language :: Python :: 3.7',
                     'Topic :: Software Development :: Testing',
                     'Topic :: Software Development :: Libraries',
                     'Topic :: Utilities',
                     'Programming Language :: Python',
                     ],
        packages=['detox'],
        install_requires=['tox>=2,<4', 'py>=1.4.27', 'eventlet>=0.15.0'],
        entry_points={'console_scripts': 'detox=detox.main:main',
                      'tox': ['proclimit = detox.tox_proclimit']},
    )


if __name__ == '__main__':
    main()



from datetime import datetime, timedelta

import pytest
import eventlet
import detox
from detox.proc import Resources

class TestResources:
    def test_getresources(self):
        l= []
        class Provider:
            def provide_abc(self):
                l.append(1)
                return 42
        resources = Resources(Provider())
        res, = resources.getresources("abc")
        assert res == 42
        assert len(l) == 1
        res, = resources.getresources("abc")
        assert len(l) == 1
        assert res == 42

    def test_getresources_param(self):
        class Provider:
            def provide_abc(self, param):
                return param
        resources = Resources(Provider())
        res, = resources.getresources("abc:123")
        return res == "123"

    def test_getresources_parallel(self):
        l= []
        queue = eventlet.Queue()
        class Provider:
            def provide_abc(self):
                l.append(1)
                return 42
        resources = Resources(Provider())
        pool = eventlet.GreenPool(2)
        pool.spawn(lambda: resources.getresources("abc"))
        pool.spawn(lambda: resources.getresources("abc"))
        pool.waitall()
        assert len(l) == 1

    def test_getresources_multi(self):
        l= []
        queue = eventlet.Queue()
        class Provider:
            def provide_abc(self):
                l.append(1)
                return 42
            def provide_def(self):
                l.append(1)
                return 23
        resources = Resources(Provider())
        a, d = resources.getresources("abc", "def")
        assert a == 42
        assert d == 23

class TestDetoxExample1:
    pytestmark = [pytest.mark.example1, pytest.mark.timeout(20)]

    def test_createsdist(self, detox):
        sdist, = detox.getresources("sdist")
        assert sdist.check()

    def test_getvenv(self, detox):
        venv, = detox.getresources("venv:py")
        assert venv.envconfig.envdir.check()
        venv2, = detox.getresources("venv:py")
        assert venv == venv2

    def test_test(self, detox):
        detox.runtests("py")

class TestDetoxExample2:
    pytestmark = [pytest.mark.example2, pytest.mark.timeout(20)]

    def test_test(self, detox):
        detox.runtests("py")

class TestCmdline:
    pytestmark = [pytest.mark.example1]
    @pytest.mark.timeout(20)
    def test_runtests(self, cmd):
        result = cmd.rundetox("-e", "py", "-v", "-v")
        result.stdout.fnmatch_lines([
            "py*getenv*",
            "py*create:*",
        ])


class TestProcLimitOption:
    pytestmark = [pytest.mark.example3]

    def test_runtestmulti(self):
        class MyConfig:
            class MyOption:
                numproc = 7
            option = MyOption()

        l = []

        def MyGreenPool(**kw):
            l.append(kw)
            # Building a Detox object will already call GreenPool(),
            # so we have to let MyGreenPool being called twice before raise
            if len(l) == 2:
                raise ValueError

        from detox import proc
        setattr(proc, 'GreenPool', MyGreenPool)
        with pytest.raises(ValueError):
            d = proc.Detox(MyConfig())
            d.runtestsmulti(['env1', 'env2', 'env3'])  # Fake env list

        assert l[0] == {}  # When building Detox object
        assert l[1] == {'size': 7}  # When calling runtestsmulti

    @pytest.mark.timeout(60)
    def test_runtests(self, cmd):
        now1 = datetime.now()
        cmd.rundetox("-n", "1", "-epy1,py2")
        then1 = datetime.now()
        delta1 = then1 - now1
        assert delta1 >= timedelta(seconds=2)

        now2 = datetime.now()
        cmd.rundetox("--num", "2", "-epy1,py2")
        then2 = datetime.now()
        delta2 = then2 - now2
        assert delta2 >= timedelta(seconds=1)

        assert delta1 >= delta2, 'pool size=2 took much time than pool size=1'
